@@ -16,7 +16,7 @@
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 # Copyright 2004 Duke University
 
-# $Id: dumpMetadata.py,v 1.39 2006/07/19 19:25:08 pnasrat Exp $
+# $Id: dumpMetadata.py,v 1.48 2007/05/18 15:01:16 pnasrat Exp $
 
 import os
 import rpm
@@ -27,6 +27,8 @@ import types
 import struct
 import re
 import stat
+import bz2
+import sqlitecachec
 
 # done to fix gzip randomly changing the checksum
 import gzip
@@ -54,7 +56,20 @@ class GzipFile(gzip.GzipFile):
 def _gzipOpen(filename, mode="rb", compresslevel=9):
     return GzipFile(filename, mode, compresslevel)
     
+def bzipFile(source, dest):
+    
+    s_fn = open(source, 'rb')
+    destination = bz2.BZ2File(dest, 'w', compresslevel=9)
 
+    while True:
+        data = s_fn.read(1024000)
+        
+        if not data: break
+        destination.write(data)
+
+    destination.close()
+    s_fn.close()
+    
 
 def returnFD(filename):
     try:
@@ -324,11 +339,12 @@ class RpmMetaData:
     
     def _stringToVersion(self, strng):
         i = strng.find(':')
-        if i != -1:
+        if i != -1 and strng[:i].isdigit():
             epoch = strng[:i]
         else:
+            i = -1
             epoch = self.noepoch
-        j = strng.find('-')
+        j = strng.rfind('-')
         if j != -1:
             if strng[i + 1:j] == '':
                 version = None
@@ -570,9 +586,16 @@ class RpmMetaData:
         if not self.options['cache']:
             return getChecksum(self.options['sumtype'], fo)
 
-        key = "".join([hex(ord(x))[2:].zfill(2)
-                       for x in tuple(self.hdr[rpm.RPMTAG_SIGMD5])])
+        t = []
+        if type(self.hdr[rpm.RPMTAG_SIGGPG]) is not types.NoneType:
+            t.append("".join(self.hdr[rpm.RPMTAG_SIGGPG]))   
+        if type(self.hdr[rpm.RPMTAG_SIGPGP]) is not types.NoneType:
+            t.append("".join(self.hdr[rpm.RPMTAG_SIGPGP]))
+        if type(self.hdr[rpm.RPMTAG_HDRID]) is not types.NoneType:
+            t.append("".join(self.hdr[rpm.RPMTAG_HDRID]))
 
+        key = md5.new("".join(t)).hexdigest()
+                                        
         csumtag = '%s-%s' % (self.hdr['name'] , key)
         csumfile = '%s/%s' % (self.options['cachedir'], csumtag)
         if os.path.exists(csumfile) and self.mtime <= os.stat(csumfile)[8]:
@@ -748,14 +771,78 @@ def repoXML(node, cmds):
     workfiles = [(cmds['otherfile'], 'other',),
                  (cmds['filelistsfile'], 'filelists'),
                  (cmds['primaryfile'], 'primary')]
+    repoid='garbageid'
     
+    repopath = os.path.join(cmds['outputdir'], cmds['tempdir'])
     
+    if cmds['database']:
+        try:
+            dbversion = str(sqlitecachec.DBVERSION)
+        except AttributeError:
+            dbversion = '9'
+        rp = sqlitecachec.RepodataParserSqlite(repopath, repoid, None)
+
     for (file, ftype) in workfiles:
-        zfo = _gzipOpen(os.path.join(cmds['outputdir'], cmds['tempdir'], file))
+        complete_path = os.path.join(repopath, file)
+        
+        zfo = _gzipOpen(complete_path)
         uncsum = getChecksum(sumtype, zfo)
         zfo.close()
-        csum = getChecksum(sumtype, os.path.join(cmds['outputdir'], cmds['tempdir'], file))
-        timestamp = os.stat(os.path.join(cmds['outputdir'], cmds['tempdir'], file))[8]
+        csum = getChecksum(sumtype, complete_path)
+        timestamp = os.stat(complete_path)[8]
+        
+        db_csums = {}
+        db_compressed_sums = {}
+        
+        if cmds['database']:
+            if ftype == 'primary':
+                rp.getPrimary(complete_path, csum)
+                            
+            elif ftype == 'filelists':
+                rp.getFilelists(complete_path, csum)
+                
+            elif ftype == 'other':
+                rp.getOtherdata(complete_path, csum)
+            
+
+            tmp_result_name = '%s.xml.gz.sqlite' % ftype
+            tmp_result_path = os.path.join(repopath, tmp_result_name)
+            good_name = '%s.sqlite' % ftype
+            resultpath = os.path.join(repopath, good_name)
+            
+            # rename from silly name to not silly name
+            os.rename(tmp_result_path, resultpath)
+            compressed_name = '%s.bz2' % good_name
+            result_compressed = os.path.join(repopath, compressed_name)
+            db_csums[ftype] = getChecksum(sumtype, resultpath)
+            
+            # compress the files
+            bzipFile(resultpath, result_compressed)
+            # csum the compressed file
+            db_compressed_sums[ftype] = getChecksum(sumtype, result_compressed)
+            # remove the uncompressed file
+            os.unlink(resultpath)
+
+            # timestamp the compressed file
+            db_timestamp = os.stat(result_compressed)[8]
+            
+            # add this data as a section to the repomdxml
+            db_data_type = '%s_db' % ftype
+            data = node.newChild(None, 'data', None)
+            data.newProp('type', db_data_type)
+            location = data.newChild(None, 'location', None)
+            if cmds['baseurl'] is not None:
+                location.newProp('xml:base', cmds['baseurl'])
+            
+            location.newProp('href', os.path.join(cmds['finaldir'], compressed_name))
+            checksum = data.newChild(None, 'checksum', db_compressed_sums[ftype])
+            checksum.newProp('type', sumtype)
+            db_tstamp = data.newChild(None, 'timestamp', str(db_timestamp))
+            unchecksum = data.newChild(None, 'open-checksum', db_csums[ftype])
+            unchecksum.newProp('type', sumtype)
+            database_version = data.newChild(None, 'database_version', dbversion)
+            
+            
         data = node.newChild(None, 'data', None)
         data.newProp('type', ftype)
         location = data.newChild(None, 'location', None)
@@ -790,3 +877,5 @@ def repoXML(node, cmds):
         checksum = data.newChild(None, 'checksum', csum)
         checksum.newProp('type', sumtype)
         timestamp = data.newChild(None, 'timestamp', str(timestamp))
+    
+        
